/*
 * $Id: LastfmTopArtists.java 362 2008-04-14 19:14:12Z jasta $
 *
 * Copyright (C) 2008 Josh Guilfoyle <jasta@devtcg.org>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

package org.devtcg.five.music.activity;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import org.devtcg.five.music.R;
import org.devtcg.five.music.lastfm.TopArtistsByUser;
import org.devtcg.five.provider.Five;
import org.devtcg.five.util.ThreadStoppable;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.DialogInterface.OnCancelListener;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewInflate;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Toast;

public class LastfmTopArtists extends Activity
{
	private static final String TAG = "LastfmTopArtists";

	/* XXX: Hardcoded! */
	private static final String USER = "jasta";

	private ProgressDialog mProgress;
	
	private ListView mList;
	
	private Cursor mCursor;
	
	private Handler mHandler = new Handler();
	private ThreadStoppable mWorker;
	
	private boolean mShutdown = false;
	
	@Override
	public void onCreate(Bundle icicle)
	{
		super.onCreate(icicle);
		setContentView(R.layout.lastfm_top_artists);

		mList = (ListView)findViewById(android.R.id.list);
		mList.setOnItemClickListener(mClickEvent);

		accessLastfm();
	}
	
	private final OnItemClickListener mClickEvent = new OnItemClickListener()
	{
		public void onItemClick(AdapterView av, View v, int pos, long id)
		{
			Uri chosen = Five.Music.Artists.CONTENT_URI.buildUpon()
			  .appendPath(String.valueOf(id))
			  .build();

			startActivity(new Intent(Intent.VIEW_ACTION, chosen,
			  LastfmTopArtists.this, AlbumAndSongList.class));
		}
	};

	private void accessLastfm()
	{
		mProgress = ProgressDialog.show(this, "Accessing Last.fm...",
		  "Please wait while Five downloads Last.fm meta data...", true, true,
		  mCanceled);

		mWorker = new ThreadStoppable()
		{
			public void run()
			{
				final TopArtistsByUser meta =
				  new TopArtistsByUser(getContentResolver(), USER, 20);

				try
				{
					meta.fetch();
				}
				catch (Exception e)
				{
					final String msg = e.toString();

					if (mStopped == true)
						return;

					mHandler.post(new Runnable() {
						public void run()
						{
							Toast.makeText(LastfmTopArtists.this, "BUG: " + msg,
							  Toast.LENGTH_SHORT).show();
						}
					});
				}

				if (mStopped == true)
					return;

				mHandler.post(new Runnable() {
					public void run()
					{
						if (meta != null)
							initFromMeta(meta);

						Log.d(TAG, "All done!");
						mProgress.dismiss();

						mWorker = null;
					}
				});
			}
		};

		mWorker.start();
	}
	
	private final OnCancelListener mCanceled = new OnCancelListener()
	{
		public void onCancel(DialogInterface i)
		{
			finish();
		}
	};

	private void shutdown()
	{
		if (mWorker == null)
			return;

		mWorker.interruptAndStop();

		while (true)
		{
			try { mWorker.join(); break; }
			catch (InterruptedException e) { }
		}

		mWorker = null;
	}

	@Override
	public void onDestroy()
	{
		Log.d(TAG, "Toodles!");
		
		shutdown();
		super.onDestroy();
	}

	private void initFromMeta(TopArtistsByUser meta)
	{
		List<TopArtistsByUser.Artist> artists = meta.getArtists();
		
		if (artists == null || artists.isEmpty() == true)
		{
			mList.setVisibility(View.GONE);
			findViewById(android.R.id.empty).setVisibility(View.VISIBLE);
		}
		else
		{
			Cursor c = expandToCursor(artists);

			assert c != null;
			assert c.count() > 0;

			SimpleCursorAdapter adapter = new SimpleCursorAdapter(this,
			  R.layout.artist_list_item, c,
			  new String[] { "photo", "full_name" },
			  new int[] { R.id.artist_photo, R.id.artist_name });
			  
			mList.setAdapter(adapter);

			mCursor = c;
		}
	}
	
	private Cursor expandToCursor(List<TopArtistsByUser.Artist> artists)
	{
		Uri uri = 
		  Uri.parse("content://org.devtcg.five/media/music/artists");

		String[] proj = {
			"_id", "name", "full_name", "photo",
		};

		Cursor c = managedQuery(uri, proj, getWhere(artists), null, null);
		
		return c;
	}
	
	private String getWhere(List<TopArtistsByUser.Artist> artists)
	{
		assert artists.isEmpty() == false;

		StringBuffer b = new StringBuffer("_id IN (");

		Iterator<TopArtistsByUser.Artist> i = artists.iterator();

		assert i.hasNext() == true;
		b.append(i.next().artistId);

		while (i.hasNext() == true)
		{
			b.append(',');
			b.append(i.next().artistId);
		}

		b.append(')');

		return b.toString();
	}

//	private static class Adapter extends BaseAdapter
//	{
//		private Context mContext;
//		private ViewInflate mInflate;
//
//		private List<TopArtistsByUser.artist> martists;
//
//		public Adapter(Context ctx, List<TopArtistsByUser.artist> artists)
//		{
//			mContext = ctx;
//			martists = artists;
//			mInflate = ViewInflate.from(ctx);
//		}
//
//		public int getCount()
//		{
//			return martists.size();
//		}
//
//		public Object getItem(int position)
//		{
//			return martists.get(position);
//		}
//
//		public long getItemId(int position)
//		{
//			return martists.get(position).artistId;
//		}
//
//		public View getView(int position, View convertView, ViewGroup parent)
//		{
//			View v;
//
//			if (convertView != null)
//				v = convertView;
//			else
//				v = mInflate.inflate(R.layout.artist_list_item, null, null);
//			
//
//			
//			return null;
//		}
//	}
}
